@@ -18,6 +18,7 @@ import functools
 import math
 import os
 import os.path
+import pyhg19
 import seqalign
 
 from multiprocessing import Pool
@@ -43,11 +44,7 @@ def setup_directory(output_dir, snp_dir):
     """
     
     for directory in (output_dir, snp_dir) + tuple(
-        os.path.join(output_dir, subdir)
-        for
-        subdir
-        in
-        (
+        os.path.join(output_dir, subdir) for subdir in (
             'map1',
             'find_intersecting_snps',
             'map2',
@@ -64,7 +61,8 @@ def map_reads(
     trim_qual,
     processes=1,
     algorithm=None,
-    reference_genome='/home/joshchiou/references/ucsc.hg19.fasta'
+    reference_genome=pyhg19.PATH,
+    temp_dir=None
 ):
     """Map some reads
     
@@ -95,7 +93,8 @@ def map_reads(
             algorithm=algorithm,
             algorithm_switch_bp=100,
             reference_genome_path=reference_genome
-        )
+        ),
+        temp_dir=temp_dir
     ) as sa:
         sa.samtools_sort()
         sa.samtools_index
@@ -108,7 +107,8 @@ def preprocess_sample(
     output_dir,
     processes=1,
     algorithm=None,
-    reference_genome='/home/joshchiou/references/ucsc.hg19.fasta'
+    reference_genome=pyhg19.PATH,
+    temp_dir=None
 ):
     """Apply preprocessing steps to an input sample
     
@@ -142,7 +142,8 @@ def preprocess_sample(
             trim_qual=15,
             processes=processes,
             algorithm=algorithm,
-            reference_genome=reference_genome
+            reference_genome=reference_genome,
+            temp_dir=temp_dir
         )
         alignment.write(map1_path)
 
@@ -152,7 +153,8 @@ def preprocessing_step(
     output_dir,
     processes=1,
     algorithm=None,
-    reference_genome='/home/joshchiou/references/ucsc.hg19.fasta'
+    reference_genome=pyhg19.PATH,
+    temp_dir=None
 ):
     """Apply preprocessing steps to all input samples
     
@@ -176,14 +178,12 @@ def preprocessing_step(
                 output_dir=output_dir,
                 processes=max(1, math.floor(processes / n_samples)),
                 algorithm=algorithm,
-                reference_genome=reference_genome
+                reference_genome=reference_genome,
+                temp_dir=temp_dir
             ),
             (
                 (sample_name, input_file_path)
-                for
-                sample_name, *input_file_path
-                in
-                sample_list
+                for sample_name, *input_file_path in sample_list
             )
         )
 
@@ -272,10 +272,7 @@ def find_intersecting_snps(
                         '{}.sort.bam'.format(sample_name)
                     )
                 )
-                for
-                sample_name, *input_file_path
-                in
-                sample_list
+                for sample_name, *input_file_path in sample_list
             )
         )
 
@@ -286,7 +283,8 @@ def remap_sample(
     output_dir,
     processes=1,
     algorithm=None,
-    reference_genome='/home/joshchiou/references/ucsc.hg19.fasta'
+    reference_genome=pyhg19.PATH,
+    temp_dir=None
 ):
     """Remap an input sample
     
@@ -310,7 +308,8 @@ def remap_sample(
         trim_qual=0,
         processes=processes,
         algorithm=algorithm,
-        reference_genome=reference_genome
+        reference_genome=reference_genome,
+        temp_dir=temp_dir
     )
     alignment.write(
         os.path.join(output_dir, 'map2', '{}.sort.bam'.format(sample_name))
@@ -322,7 +321,8 @@ def remapping_step(
     output_dir,
     processes=1,
     algorithm=None,
-    reference_genome='/home/joshchiou/references/ucsc.hg19.fasta'
+    reference_genome=pyhg19.PATH,
+    temp_dir=None
 ):
     """Remap all input samples
     
@@ -346,7 +346,8 @@ def remapping_step(
                 output_dir=output_dir,
                 processes=max(1, math.floor(processes / n_samples)),
                 algorithm=algorithm,
-                reference_genome=reference_genome
+                reference_genome=reference_genome,
+                temp_dir=temp_dir
             ),    
             (
                 (
@@ -361,25 +362,17 @@ def remapping_step(
                                     end
                                 )
                             )
-                            for
-                            end
-                            in
-                            (1, 2)
+                            for end in (1, 2)
                         )
-                        if
-                        len(input_file_path) == 2
-                        else
-                        os.path.join(
+                        if len(input_file_path) == 2
+                        else os.path.join(
                             output_dir,
                             'find_intersecting_snps',
                             '{}.sort.remap.fq.gz'.format(sample_name)
                         )
                     )
                 )
-                for
-                sample_name, *input_file_path
-                in
-                sample_list
+                for sample_name, *input_file_path in sample_list
             )
         )
 
@@ -425,15 +418,17 @@ def filter_remapped_reads(sample_list, output_dir, processes=1, memory_limit=5):
                         '{}.keep.bam'.format(sample_name)
                     )
                 )
-                for
-                sample_name, *input_file_path
-                in
-                sample_list
+                for sample_name, *input_file_path in sample_list
             )
         )
 
 
-def merge_and_rmdup(*sequence_alignments, paired_end=False, processes=1):
+def merge_and_rmdup(
+    *sequence_alignments,
+    paired_end=False,
+    processes=1,
+    temp_dir=None
+):
     """Merge kept and remapped reads and apply WASP's dedupper
     
     Parameters
@@ -449,7 +444,8 @@ def merge_and_rmdup(*sequence_alignments, paired_end=False, processes=1):
     sa = seqalign.merge(
         *sequence_alignments,
         processes=processes,
-        dedupper=RmDup(paired_end=paired_end, processes=processes)
+        dedupper=RmDup(paired_end=paired_end, processes=processes),
+        temp_dir=temp_dir
     )
     sa.samtools_sort()
     sa.samtools_index()
@@ -466,7 +462,8 @@ def merge_rmdup_pileup(
     output_dir,
     snp_dir,
     reference_genome_path,
-    processes=1
+    processes=1,
+    temp_dir=None
 ):
     """Merge kept and remapped reads, remove duplicates, and generate a pileup
     
@@ -500,7 +497,8 @@ def merge_rmdup_pileup(
             '{}.sort.keep.bam'.format(sample_name)
         ),
         paired_end=len(input_file_path) == 2,
-        processes=processes
+        processes=processes,
+        temp_dir=temp_dir
     )
     with open(
         os.path.join(
@@ -524,7 +522,8 @@ def merge_rmdup_pileup_steps(
     output_dir,
     snp_dir,
     reference_genome_path,
-    processes=1
+    processes=1,
+    temp_dir=None
 ):
     """Merge reads, remove duplicates, and generate a pileup for all samples
     
@@ -550,14 +549,12 @@ def merge_rmdup_pileup_steps(
                 output_dir=output_dir,
                 snp_dir=snp_dir,
                 reference_genome_path=reference_genome_path,
-                processes=max(1, math.floor(processes / n_samples))
+                processes=max(1, math.floor(processes / n_samples)),
+                temp_dir=temp_dir
             ),
             (
                 (sample_name, input_file_path)
-                for
-                sample_name, *input_file_path
-                in
-                sample_list
+                for sample_name, *input_file_path in sample_list
             )
         )
 
@@ -575,14 +572,8 @@ def remove_intermediate_files(samples, output_dir):
     
     for file_path in (
         file_path
-        for
-        sample_name
-        in
-        (sample[0] for sample in samples)
-        for
-        file_path
-        in
-        (
+        for sample_name in (sample[0] for sample in samples)
+        for file_path in (
             os.path.join(
                 output_dir,
                 'map2',
@@ -617,10 +608,7 @@ def remove_intermediate_files(samples, output_dir):
                     end
                 )
             )
-            for
-            end
-            in
-            (1, 2)
+            for end in (1, 2)
         )
         if
         os.path.isfile(file_path)
@@ -704,10 +692,9 @@ def parse_arguments():
     config_group.add_argument(
         '--reference-genome',
         metavar='<path/to/reference_genome.fa>',
-        default='/home/joshchiou/references/ucsc.hg19.fasta',
+        default=pyhg19.PATH,
         help=(
-            'Path to reference genome '
-            '[/home/joshchiou/references/ucsc.hg19.fasta]'
+            f'Path to reference genome [{pyhg19.PATH}]'
         )
     )
     config_group.add_argument(
@@ -726,13 +713,18 @@ def parse_arguments():
         choices={'aln', 'mem'},
         help='Force BWA algorithm'
     )
+    config_group.add_argument(
+        '--skip-to-4',
+        action='store_true',
+        help='skip to step 4 (remapping)'
+    )
     resource_group = parser.add_argument_group('resource arguments')
     resource_group.add_argument(
         '--processes',
         metavar='<int>',
-        default=1,
+        default=2,
         type=int,
-        help='Maximum number of processes allowed [1]'
+        help='Maximum number of processes allowed. Must be >= 2. [2]'
     )
     resource_group.add_argument(
         '--memory-limit',
@@ -741,7 +733,14 @@ def parse_arguments():
         type=float,
         help='Approximate memory limit in gigabytes [5]'
     )
+    resource_group.add_argument(
+        '--tmp-dir',
+        metavar='<path/to/tmp/>',
+        help='directory for temporary files'
+    )
     args = parser.parse_args()
+    if args.processes < 2:
+        raise RuntimeError('allow at least 2 processes')
     if not args.snp_dir:
         args.snp_dir = os.path.join(args.output_dir, 'snp_dir')
     return args
@@ -774,24 +773,27 @@ def main():
     
     # Stop here if no sample was given
     if args.sample:
-    
-        # Step 2: preprocess sequencing data
-        preprocessing_step(
-            args.sample,
-            args.output_dir,
-            processes=args.processes,
-            algorithm=args.algorithm,
-            reference_genome=args.reference_genome
-        )
-    
-        # Step 3: find intersecting snps
-        find_intersecting_snps(
-            args.sample,
-            args.output_dir,
-            args.snp_dir,
-            processes=args.processes,
-            memory_limit=args.memory_limit
-        )
+        
+        if not args.skip_to_4:
+
+            # Step 2: preprocess sequencing data
+            preprocessing_step(
+                args.sample,
+                args.output_dir,
+                processes=args.processes,
+                algorithm=args.algorithm,
+                reference_genome=args.reference_genome,
+                temp_dir=args.tmp_dir
+            )
+        
+            # Step 3: find intersecting snps
+            find_intersecting_snps(
+                args.sample,
+                args.output_dir,
+                args.snp_dir,
+                processes=args.processes,
+                memory_limit=args.memory_limit
+            )
     
         # Step 4: remap
         remapping_step(
@@ -799,7 +801,8 @@ def main():
             args.output_dir,
             processes=args.processes,
             algorithm=args.algorithm,
-            reference_genome=args.reference_genome
+            reference_genome=args.reference_genome,
+            temp_dir=args.tmp_dir
         )
     
         # Step 5: filter remapped reads
@@ -816,29 +819,8 @@ def main():
             args.output_dir,
             args.snp_dir,
             args.reference_genome,
-            processes=args.processes
-        )
-    
-        # Clean up intermediate files
-        if not args.save_intermediate:
-            remove_intermediate_files(args.sample, args.output_dir)
-        
-    # Clean up empty directories
-    if not args.save_intermediate:
-        remove_empty_directories(args.output_dir)
-
-
-def main_skip_to_6(args):
-    # Stop here if no sample was given
-    if args.sample:
-    
-        # Step 6-7+: Merge remapped_reads, remove duplicates, generate pileups
-        merge_rmdup_pileup_steps(
-            args.sample,
-            args.output_dir,
-            args.snp_dir,
-            args.reference_genome,
-            processes=args.processes
+            processes=args.processes,
+            temp_dir=args.tmp_dir
         )
     
         # Clean up intermediate files
